#basic cleaning
rm(list=ls()) #variables
if (!is.null(dev.list()["RStudioGD"])){
  dev.off(dev.list()["RStudioGD"])} #previous plots

#reqs
require(dplyr)||install.packages(dplyr);
require(rgl)||install.packages("rgl");
require(tidyverse)||install.packages("tidyverse");
require(ggplot2)||install.packages("ggplot2");
#require(hrbrthemes)||install.packages("hrbrthemes")
require(viridis)||install.packages("viridis")


experimentRecord_name="Exp_DualSourcing_07"; #base record of experiments
showPareto_byExpID <- NA #vector of number(s), or NA for none
boxplot_HV_timeLimit <- NA #vector of number(s), or NA for none

load(experimentRecord_name);
#load("KeyRing")
load("fileOfMSOprototypeFunctions")
df <- record_experiments$df;
df=df[!is.na(df$FullEvaluationTime),];
df$TimeToSolution <- df$FullEvaluationTime + df$SurrogateOptimizerTime;
df$CrowdingDistance <- df$CrowdingDistance/record_experiments$perfectCrowdingDistance;



#let's group our experiments up
stdDev_confidence=1.9723;
df_grouped <- df %>% group_by(UnderlyingModel, SurrogateType, Optimizer, InitialSample,TimeLimit) %>%
 dplyr::summarize(HV_mean = mean(HV_counted),
                  HV_CIplusminus = stdDev_confidence*sd(HV_counted)/sqrt(sum(!is.na(FullEvaluationRuns))),
                  Hausdoff_mean = mean(Hausdoff),
                  Hausdoff_CIplusminus = stdDev_confidence*sd(Hausdoff)/sqrt(sum(!is.na(FullEvaluationRuns))),
                  Crowding_mean=mean(CrowdingDistance),
                  Crowding_CIplusminus = stdDev_confidence*sd(CrowdingDistance)/sqrt(sum(!is.na(FullEvaluationRuns)))
                  ) %>% as.data.frame()

#we need to add a MOpair to each entry
findShortForm <- function(string){
  if (string=="GaussianRegression"){return("GR")}
  if (string=="neuralnet"){return("NN")}
  if (string=="Basic"){return("B")}
  if (string=="GeneticAlgorithm1"){return("GA")}
}

df$MOpair <- NA;
df_grouped$MOpair <- NA;
for (c1 in 1:nrow(df)){
  df$MOpair[c1] <- paste(c(findShortForm(df$SurrogateType[c1]),"-",
                       findShortForm(df$Optimizer[c1])),collapse="");
}
rm(c1);
for (c1 in 1:nrow(df_grouped)){
  df_grouped$MOpair[c1] <- paste(c(findShortForm(df_grouped$SurrogateType[c1]),"-",
                           findShortForm(df_grouped$Optimizer[c1])),collapse="");
}
rm(c1);


pchs=c(16,17,4,18,6:10)
colors=c("red","black","blue","green","yellow");
CI_lines=c(2:6);

df_grouped$HV_mean_upper <- df_grouped$HV_mean + df_grouped$HV_CIplusminus;
df_grouped$HV_mean_lower <- df_grouped$HV_mean - df_grouped$HV_CIplusminus;
df_grouped$Hausdoff_mean_upper <- df_grouped$Hausdoff_mean + df_grouped$Hausdoff_CIplusminus;
df_grouped$Hausdoff_mean_lower <- df_grouped$Hausdoff_mean - df_grouped$Hausdoff_CIplusminus;
df_grouped$Crowding_mean_upper <- df_grouped$Crowding_mean + df_grouped$Crowding_CIplusminus;
df_grouped$Crowding_mean_lower <- df_grouped$Crowding_mean - df_grouped$Crowding_CIplusminus;

TimeLimit_lim=c(min(df_grouped$TimeLimit),max(df_grouped$TimeLimit));
HV_lim=c(min(df_grouped$HV_mean_lower),max(df_grouped$HV_mean_upper));
Hausdoff_lim=c(min(df_grouped$Hausdoff_mean_lower),max(df_grouped$Hausdoff_mean_upper));
Crowding_lim=c(min(df_grouped$Crowding_mean_lower),max(df_grouped$Crowding_mean_upper));


#hypervolume plot
#set the plot for the first "class"
classes <- unique(df_grouped$MOpair);
selected <- df_grouped[df_grouped$MOpair==classes[1],];
TimeLimits=unique(selected$TimeLimit);
par(xaxt="n")
plot(selected$TimeLimit, selected$HV_mean,
     xlim=TimeLimit_lim, ylim=HV_lim,
     pch=pchs[1], col=colors[1], type="p",
     xlab="Time Limit (s)",
     ylab="Normalized Hypervolume",
     main=df_grouped$UnderlyingModel[1]);
par(xaxt="s")
axis(side=1,at=TimeLimits,labels=TimeLimits)
legend_strings=c();
for (c1 in 1:length(classes)){
  selected <- df_grouped[df_grouped$MOpair==classes[c1],];
  points(selected$TimeLimit,selected$HV_mean,pch=pchs[c1],col=colors[c1]);
  lines(selected$TimeLimit,selected$HV_mean_upper,col=colors[c1],type="l",lty=CI_lines[c1]);
  lines(selected$TimeLimit,selected$HV_mean_lower,col=colors[c1],type="l",lty=CI_lines[c1]);
  legend_strings <- c(legend_strings,classes[c1]);
}
legend("bottomright",legend=legend_strings,pch=pchs[1:length(classes)],col=colors[1:length(classes)],lty=CI_lines[1:length(classes)]);


selected <- df_grouped[df_grouped$MOpair==classes[1],];
TimeLimits=unique(selected$TimeLimit);
par(xaxt="n")
plot(selected$TimeLimit, selected$Hausdoff_mean,
     xlim=TimeLimit_lim, ylim=Hausdoff_lim,
     pch=pchs[1], col=colors[1], type="p",
     xlab="Time Limit (s)",
     ylab="Normalized Hausdorff Distance",
     main=df_grouped$UnderlyingModel[1]);
par(xaxt="s")
axis(side=1,at=TimeLimits,labels=TimeLimits)
legend_strings=c();
for (c1 in 1:length(classes)){
  selected <- df_grouped[df_grouped$MOpair==classes[c1],];
  points(selected$TimeLimit,selected$Hausdoff_mean,pch=pchs[c1],col=colors[c1]);
  lines(selected$TimeLimit,selected$Hausdoff_mean_upper,col=colors[c1],type="l",lty=CI_lines[c1]);
  lines(selected$TimeLimit,selected$Hausdoff_mean_lower,col=colors[c1],type="l",lty=CI_lines[c1]);
  legend_strings <- c(legend_strings,classes[c1]);
}
legend("topright",legend=legend_strings,pch=pchs[1:length(classes)],col=colors[1:length(classes)],lty=CI_lines[1:length(classes)]);


selected <- df_grouped[df_grouped$MOpair==classes[1],];
TimeLimits=unique(selected$TimeLimit);
par(xaxt="n")
plot(selected$TimeLimit, selected$Crowding_mean,
     xlim=TimeLimit_lim, ylim=Crowding_lim,
     pch=pchs[1], col=colors[1], type="p",
     xlab="Time Limit (s)",
     ylab="Normalized Crowding Distance",
     main=df_grouped$UnderlyingModel[1]);
par(xaxt="s")
axis(side=1,at=TimeLimits,labels=TimeLimits)
legend_strings=c();
for (c1 in 1:length(classes)){
  selected <- df_grouped[df_grouped$MOpair==classes[c1],];
  points(selected$TimeLimit,selected$Crowding_mean,pch=pchs[c1],col=colors[c1]);
  lines(selected$TimeLimit,selected$Crowding_mean_upper,col=colors[c1],type="l",lty=CI_lines[c1]);
  lines(selected$TimeLimit,selected$Crowding_mean_lower,col=colors[c1],type="l",lty=CI_lines[c1]);
  legend_strings <- c(legend_strings,classes[c1]);
}
legend("topright",legend=legend_strings,pch=pchs[1:length(classes)],col=colors[1:length(classes)],lty=CI_lines[1:length(classes)]);








load(findAnswerKey(record_experiments$df$UnderlyingModel[1],"KeyRing"));
perfectPareto <- AnswerKey$paretoFront;

xlim=c(min(perfectPareto$Output1),max(perfectPareto$Output1));
ylim=c(min(perfectPareto$Output2),max(perfectPareto$Output2));
if (AnswerKey$outputNum>2){
 zlim= c(min(perfectPareto$Output3),max(perfectPareto$Output3));
  
}

dominatedPolygon = function(xlim,ylim,paretoFront,Output_max){
  order2 <- paretoFront$Output2;
  if (!Output_max[2]){
   order2<- -order2 
  }
  paretoFront <- paretoFront[order(paretoFront$Output1,order2),c("Output1","Output2")];
  rownum=nrow(paretoFront);
  paretoFront[paretoFront$Output1<xlim[1],"Output1"] <- xlim[1];
  paretoPolygon <- paretoFront;
  row_counter <- 1;
  #first point, origin
  #depends on the min max, etc
  #default to the case of max
  x_toAdd <- xlim[1];
  y_toAdd <- ylim[1];
  
  if(!Output_max[1]){
   x_toAdd <- xlim[2];
  }
  if(!Output_max[2]){
   y_toAdd <- ylim[2]; 
  }
  
  paretoPolygon[row_counter,c("Output1","Output2")] <- c(x_toAdd,y_toAdd);
  
  #points in list
  
  if (Output_max[1] & Output_max[2]){
    for (c1 in 1:rownum){
      row_counter <- row_counter+1;
      y_toAdd <- paretoFront$Output2[c1];
      paretoPolygon[row_counter,c("Output1","Output2")] <- c(x_toAdd,y_toAdd);
      row_counter <- row_counter+1;
      x_toAdd <- paretoFront$Output1[c1]
      paretoPolygon[row_counter,c("Output1","Output2")] <- c(x_toAdd,y_toAdd);
    }
    row_counter <- row_counter+1;
    y_toAdd <- ylim[1]
    paretoPolygon[row_counter,c("Output1","Output2")] <- c(x_toAdd,y_toAdd);
    row_counter <- row_counter+1;
    x_toAdd <- xlim[1];
    paretoPolygon[row_counter,c("Output1","Output2")] <- c(x_toAdd,y_toAdd);
  }
  
  if (!Output_max[1] & !Output_max[2]){
    for (c1 in 1:rownum){
      row_counter <- row_counter+1;
      x_toAdd <- paretoFront$Output1[c1]
      paretoPolygon[row_counter,c("Output1","Output2")] <- c(x_toAdd,y_toAdd);
      row_counter <- row_counter+1;
      y_toAdd <- paretoFront$Output2[c1];
      paretoPolygon[row_counter,c("Output1","Output2")] <- c(x_toAdd,y_toAdd);
    }
    x_toAdd <- xlim[2];
    paretoPolygon[row_counter,c("Output1","Output2")] <- c(x_toAdd,y_toAdd);
    row_counter <- row_counter+1;
    y_toAdd <- ylim[2]; 
    paretoPolygon[row_counter,c("Output1","Output2")] <- c(x_toAdd,y_toAdd);
  }
  return(paretoPolygon);
}



#2-d plots
polygon_df_perfectPareto <- dominatedPolygon(xlim,ylim,perfectPareto,AnswerKey$Output_max);
for (c1 in showPareto_byExpID){
  if (is.na(c1)){
    break
  }
  rowInDf <- df$ExpID == showPareto_byExpID;
  MOpair <- df$MOpair[rowInDf];
  TimeLimit <- df$TimeLimit[rowInDf]
  Model <- df$UnderlyingModel[rowInDf]
  HV_val <- df$HV_counted[rowInDf];
  HV_val <- round(HV_val,digits=3);
  string=c(Model,", ", "TimeLimit: ",TimeLimit,"s\nNormalized Hypervolume: ",HV_val);
  string<- paste(string,collapse="");
  
  xlab=NA;
  if (Model=="CNTNV"){
    xlab="Profit";
    ylab="Quantity Sold";
    density=5;
  }
  if (Model=="StudentServices"){
    xlab="Employee Wages (units)";
    ylab="Student Time Cost (min)";
    density=10;
  }
  
  if (is.na(xlab)){
   break 
  }
  
  
  plot(perfectPareto$Output1, perfectPareto$Output2,
       xlim=xlim,ylim=ylim,
       xlab=xlab,
       ylab=ylab,
       type="p",
       pch=1, col=colors[1],
       main=string)
  
  solutionPareto <- record_experiments$paretoFronts[[c1]];
  points(solutionPareto$Output1,solutionPareto$Output2,
         type="p",
         pch=pchs[3],col=colors[2])
  

  shadex=polygon_df_perfectPareto$Output1;
  shadey=polygon_df_perfectPareto$Output2;
  
  polygon(shadex, shadey,
          col = colors[1], density=density)
  
  polygon_df <- dominatedPolygon(xlim,ylim,solutionPareto,AnswerKey$Output_max);
  shadex=polygon_df$Output1;
  shadey=polygon_df$Output2;
  polygon(shadex, shadey,
          col = colors[2], density=density, angle=135)
  #specificPareto <- paste(c("Best Pareto Front (",MOpair,")"),collapse="");
  specificPareto <- paste(c("Best ",MOpair," Pareto Front"),collapse="");
  legend_strings=c("Optimal Pareto Front",specificPareto);
  legend("topright",legend=legend_strings,
         pch=c(1,pchs[3]),
         col=colors[1:2],lty=c(1,1));
}

#3d plots
for (c1 in showPareto_byExpID){
  if (is.na(c1)){
    break
  }
  rowInDf <- df$ExpID == showPareto_byExpID;
  MOpair <- df$MOpair[rowInDf];
  TimeLimit <- df$TimeLimit[rowInDf]
  Model <- df$UnderlyingModel[rowInDf]
  HV_val <- df$HV_counted[rowInDf];
  HV_val <- round(HV_val,digits=3);
  string=c(Model,", ", "TimeLimit: ",TimeLimit,"s, Relative Hypervolume: ",HV_val);
  string<- paste(string,collapse="");
  
  xlab=NA;
  if (Model=="Telecom"){
    xlab="Voice Revenue";
    ylab="Service Revenue";
    zlab="Marketing Aggressiveness";
    radius=200;
  }
  if (Model=="DualSourcing"){
    xlab="Average Ordering Cost";
    ylab="Average Penalty";
    zlab="Average Holding Cost";
    radius=10;
  }
  
  if (is.na(xlab)){
    break 
  }
  

  solutionPareto <- record_experiments$paretoFronts[[c1]]
  solutionPareto$colors<-colors[2];
  perfectPareto$colors<-colors[1]
 # open3d()
 # bg3d(color = "white")
  plot3d( 
    x=c(solutionPareto$Output1, perfectPareto$Output1),
    y=c(solutionPareto$Output2, perfectPareto$Output2),
    z=c(solutionPareto$Output3, perfectPareto$Output3),
   xlim=xlim,ylim=ylim,zlim=zlim,
    xlab=xlab,
    ylab=ylab,
    zlab=zlab,
   col=c(solutionPareto$colors,perfectPareto$colors),
    type = 's', 
    radius = radius,
   axes = FALSE
   )
  
  xaxis<- seq.int(from=xlim[1],to=xlim[2],length.out=3)
  xaxis<-signif(xaxis, digits = 2)
  axis3d("x", at=xaxis, labels=xaxis)
  yaxis<- seq.int(from=ylim[1],to=ylim[2],length.out=3)
  yaxis<-signif(yaxis, digits = 2)
  axis3d("y", at=yaxis, labels=yaxis)
  zaxis<- seq.int(from=zlim[1],to=zlim[2],length.out=3)
  zaxis<-signif(zaxis, digits = 2)
  axis3d("z", at=zaxis, labels=zaxis)
  

  specificPareto <- paste(c("Best Pareto Front, (",MOpair,")"),collapse="");
  legend_strings=c("Optimal Pareto Front",specificPareto);
  legend3d("top",legend=legend_strings,
         pch=c(pchs[1],pchs[1]),
         col=colors[1:2],
         inset=0);
}


for (c1 in boxplot_HV_timeLimit){
  if (is.na(c1)){
    break
  }

  selection <- df[df$TimeLimit==boxplot_HV_timeLimit,];
  Model <- selection$UnderlyingModel[1]
  string <-paste(c(Model,", Time Limit: ",c1,"s"),collapse="");
  
  plotToPrint<-  ggplot(data=selection, aes(x=MOpair, y=HV_counted)) +
    geom_boxplot() +
    scale_fill_viridis(discrete = TRUE, alpha=0.6) +
    geom_jitter(color="black", size=0.85, alpha=0.9) +
    ggtitle(string) +
    xlab("MO Pair") +
    ylab("Normalized Hypervolume")
  
  print(plotToPrint)
  
  
}

