#basic cleaning
rm(list=ls()) #variables
if (!is.null(dev.list()["RStudioGD"])){
  dev.off(dev.list()["RStudioGD"])} #previous plots

## identify the files
record_name <- "Exp_CNTNV_08";
KeyRing_name <- "KeyRing";
num_countingGrid <- 10000; #points in the detailed grid-counting hypervolume calculation
performCounting <- TRUE;


require('ecr')||install.packages('ecr')
require(dplyr)||install.packages(dplyr);
load(record_name);
string=paste(c(record_name,"_backup"),collapse="");
save(record_experiments,file=string); #backup just in case
load("fileOfMSOprototypeFunctions");
load(findAnswerKey(record_experiments$df$UnderlyingModel[1],KeyRing_name));

output_max <- AnswerKey$Output_max;
inputNum <- AnswerKey$inputNum;
outputNum <- AnswerKey$outputNum;

df <- record_experiments$df;
df=df[!is.na(df$FullEvaluationTime),];
df$algorithm=NA;

#generate vector of Output column names
obj.cols <- c("Output1");
for (c1 in 2:AnswerKey$outputNum){
 string=paste(c("Output",c1),collapse=""); 
  obj.cols <- c(obj.cols,string);
}

minmax_list=list();
for (c1 in 1:length(obj.cols))
{
 minmax_list[[c1]]=c(min(AnswerKey$paretoFront[,obj.cols[c1]]),
                     max(AnswerKey$paretoFront[,obj.cols[c1]])) 
}

###### not used, probably not correct
###### the reference set, in matrix form
##ref.sets=list()
## refMatrix <- data.matrix(AnswerKey$paretoFront[,obj.cols]);
##stringToEval=paste(c("ref.sets$",record_experiments$df$UnderlyingModel[1],"=refMatrix"),collapse="")
##eval(parse(text=stringToEval));

#the indexing in the pareto fronts list is according to the ExpID (listed in df)
df_toAdd=function(experimentNum){
  #need to have columns named "prob", "algorithm" and "repl"
  if (experimentNum==0){
    df<- AnswerKey$paretoFront[,obj.cols];
    df$prob <- record_experiments$df$UnderlyingModel[1];
    df$algorithm <- "Perfect Pareto Front";
    df$repl <- 1;
  }
  else{
    ExpID <- df$ExpID[experimentNum];
    df <- record_experiments$paretoFronts[[ExpID]][,obj.cols];
    df$prob <- record_experiments$df$UnderlyingModel[ExpID];
    
    df$algorithm <- paste(c(record_experiments$df$SurrogateType[ExpID],"-",
                            record_experiments$df$Optimizer[ExpID],"-",
                            "InitSam:",record_experiments$df$InitialSample[ExpID],"-",
                            "TimeLimit:",record_experiments$df$TimeLimit[ExpID]),collapse="");
    df$repl <- record_experiments$df$Replication[ExpID];
  }
  return(df);
}

matrix_forEval <- function(df){
  #scaling
  for (c1 in 1:length(obj.cols))
  {
    vector <- df[,obj.cols[c1]];
    vector <- (vector-minmax_list[[c1]][1])/(minmax_list[[c1]][2]-minmax_list[[c1]][1])
    df[,obj.cols[c1]] <- vector;
  }
 #takes a dataframe (pareto front) and converts into format for ECR
  matrix <- data.matrix(df[,obj.cols]);
  matrix <- t(matrix);
  return(matrix);
}

#average crowding distance, from a matrix
computeMeanCrowdingDistance <- function (matrix){
  vector <- computeCrowdingDistance(matrix);
  vector <- vector[vector != Inf ];
  return(mean(vector))
}

matrix_perfect <- matrix_forEval(AnswerKey$paretoFront);
df$Hausdoff<-NA;
df$CrowdingDistance <- NA;

#initialize entry 1
df_toEval <- df_toAdd(1);
df$algorithm[1] <- df_toEval$algorithm[1];
matrix <- matrix_forEval(df_toEval);
df$Hausdoff[1] <- computeAverageHausdorffDistance(matrix,matrix_perfect,normalize=T);
df$CrowdingDistance[1] <- computeMeanCrowdingDistance(matrix)


for (c1 in 2:nrow(df)){
  addThis <- df_toAdd(c1);
  df_toEval = rbind(df_toEval,addThis);
  df$algorithm[c1] <- addThis$algorithm[1];
  matrix <- matrix_forEval(addThis);
  df$Hausdoff[c1] <- computeAverageHausdorffDistance(matrix,matrix_perfect,normalize=T);
  df$CrowdingDistance[c1] <- computeMeanCrowdingDistance(matrix);
}

#we need to scale the Hausdoff distance as well (even if it's normalized, and 0 is "perfect")
df$Hausdoff <- df$Hausdoff/max(df$Hausdoff);


#add the perfect pareto front at bottom
  df_toEval_perfect <- df_toAdd(0);
for (c1 in 1:max(as.numeric(unique(df$Replication)))){
  df_toEval_perfect$repl <- c1;
df_toEval=rbind(df_toEval,df_toEval_perfect);

}

maxes <- numeric(length(AnswerKey$Output_max));
for (c1 in 1:length(AnswerKey$Output_max)){
  if (!AnswerKey$Output_max[c1]){
    #if this was to be minimized, we need to flip-take the negative
    string=paste(c("Output",c1),collapse="");
    maxVal<- max(df_toEval[,string]);
    df_toEval[,string] <-maxVal -df_toEval[,string];
    df_toEval_perfect[,string] <- maxVal - df_toEval_perfect[,string];
  }
  #now we are all in "max"
  condToRemove = df_toEval[,string] < min(df_toEval_perfect[,string]);
  condToRemove = condToRemove + (df_toEval[,string] > max(df_toEval_perfect[,string]));
  
  #now we need to remove rows which are way off the corner
  if(sum(condToRemove)>0){
    df_toEval <- df_toEval[condToRemove==0,];
  }
  maxes[c1] <- min(df_toEval[,string])
}

print("Now computing HV")
results <- computeIndicators(df_toEval, obj.cols=obj.cols, normalize=T);
useTheseResults<-results$unary[!is.na(results$unary$repl),];
#scale these here
max=max(useTheseResults$HV);
useTheseResults$HV <- useTheseResults$HV/max;

df$HV_ecr = NA;
#fill appropriately
for (c1 in 1:nrow(df)){
  rowFound= (useTheseResults$repl == df$Replication[c1]);
  rowFound= (rowFound * (useTheseResults$algorithm == df$algorithm[c1]))
  df$HV_ecr[c1] <- useTheseResults$HV[rowFound==1];
}
#this one ignores the last output, which has our perfect HV
perfectHV_ecr <- useTheseResults$HV[useTheseResults$algorithm == "Perfect Pareto Front"];
perfectCrowdingDistance <- computeMeanCrowdingDistance(matrix_perfect)

record_experiments$perfectHV_ecr <- perfectHV_ecr;
record_experiments$perfectCrowdingDistance <- perfectCrowdingDistance;

record_experiments$df <- df;
save(record_experiments,file=record_name);


if (performCounting)
{
  #we want to calculate directly the hypervolume
  #HV_counting
  nGrid_perObjective <- num_countingGrid^(1/outputNum);
  evalVectors <- list();
  for (c1 in 1:length(obj.cols)){
    min <- min(AnswerKey$paretoFront[,obj.cols[c1]]);
    max <- max(AnswerKey$paretoFront[,obj.cols[c1]]);
    evalVectors[[c1]] <- seq(from=min,to=max,length.out=nGrid_perObjective);
  }
  evalGrid <- expand.grid(evalVectors);
  names(evalGrid) <- obj.cols;
  countedPoints <- numeric(nrow(record_experiments$df));
  for (c1 in 1:nrow(record_experiments$df)){
    #proceed to count points
    copiedGrid <- evalGrid;
    for (c2 in nrow(evalGrid):1){
      if(!dominatedByList(copiedGrid[c2,],
                          record_experiments$paretoFronts[[record_experiments$df$ExpID[c1]]][,obj.cols]))
      {
        copiedGrid <- copiedGrid[-c2,];
      }
    }
    countedPoints[c1]<-nrow(copiedGrid);
    
    if (c1%%20==0){print(paste(c("Evaluated front:",c1),collapse=" "));}
  }
  
  #for perfect Pareto front
  copiedGrid <- evalGrid;
  for (c2 in nrow(evalGrid):1){
    if(!dominatedByList(copiedGrid[c2,],
                        AnswerKey$paretoFront[,obj.cols]))
    {
      copiedGrid <- copiedGrid[-c2,];
    }
  }
  perfect_countedPoints <- nrow(copiedGrid);
  df$HV_counted <- countedPoints/perfect_countedPoints;
}

record_experiments$df <- df;
save(record_experiments,file=record_name);


