
#MANUALLY SAVE THE ANSWERKEY AT THE END
#Setting it to autosave easily causes errors overwriting by accidentally running the file

#Each entry will need
# df : data drame of Inputs (in order), Outputs (in order) and FullEvalTime
# Name (also used as the ID)
# FullName: Optional, the full name if Name isn't clear
# ExternalProgram: Where the simulation is done
# inputNum: number of inputs
# outputNum: number of outputs
# Output_max: vector (use c( ) ) of T/F (maximize this output?)
# ParetoFront (generate this from the df)
# InputLimits (a list of vectors of the min and max one for each input --
#             Inputs MUST be integers and exactly 1 apart
#             The last value of this will not be used by the solver, remember this when setting up AnswerKey)

# Notes : Note the scaling of inputs as necessary eg: "Input1/100 is price in pounds"




rm(list=ls()) #variables



AnswerKey=list();
AnswerKey$Name <- "CNTNV";
AnswerKey$FullName <- "Continuous Newsvendor Problem"
AnswerKey$ExternalProgram <- "SimOpt"
inputNum <- 2;
AnswerKey$inputNum <- inputNum; 
outputNum <- 2;
AnswerKey$outputNum <- outputNum;
output_max <- c(T,T);
AnswerKey$Output_max <-output_max;
AnswerKey$InputLimits <- list(c(80,101), c(10,41) );#remember the last entry isn't expected by solver
AnswerKey$Notes$InputNotes <- 
  "Input1/10 is sell price (no units in model) ie: from 7 to 11, Input2/100 is quantity (no units in model) ie: from 0 to 0.5";
AnswerKey$Notes$OutputNotes <- 
  "Output1 is profit (no units in model), Output2 is quantity sold (ie: the lower of Input2/100 and demand)";

replicationNum <- 1000; #time to simulate for 1 evaluation
AnswerKey$Notes$Replications <- replicationNum;

#prepping df
df <- expand.grid((AnswerKey$InputLimits[[1]][1]):(AnswerKey$InputLimits[[1]][2]-1),
                  (AnswerKey$InputLimits[[2]][1]):(AnswerKey$InputLimits[[2]][2]-1),KEEP.OUT.ATTRS= FALSE)

names(df)[1]="Input1"; #related to price
names(df)[2]="Input2"; #related to quantity

df[,c("Output1","Output2","FullEvalTime")]=NA;

#prep Python

library(reticulate) 

use_virtualenv("r-reticulate")
# virtualenv_list()
# py_config()

# IF THIS STOPS WORKING
# CHECK THE ASSOCIATED CONFIG FILE
# COMMENT OUT THE (unneeded "import imp" section)

#all things loaded now
py_run_string("import simopt")
py_run_string("from simopt import models, solvers, experiment_base")
py_run_string("from mrg32k3a.mrg32k3a import MRG32k3a")
py_run_string("simulation=simopt.models.cntnv.CntNV(fixed_factors=None)")

# py_run_string("print(simulation.factors)"); #to check the factors

#these will be fixed, call once and store
cost <- py$simulation$factors$purchase_price;
salvage <- py$simulation$factors$salvage_price;
#loop starts here

for (c1 in 1:nrow(df)){
  #for each row in the df, we need an evaluation, the mean from replicationNum trials
  
  
  
  
  time <- system.time({
    #send model inputs to python
    price <- df$Input1[c1]/10
    py$sales_price <- price; 
    py_run_string("simulation.factors['sales_price']=sales_price")
    qty <- df$Input2[c1]/100;
    py$order_quantity <- qty;
    py_run_string("simulation.factors['order_quantity']=order_quantity")
  
    storeOutput1Sum <- 0;
    storeOutput2Sum <- 0;
    for (c2 in 1:replicationNum){
     #do replications 
      
      #set up the needed RNG 
      py$s_expNum <- (c1-1);
      py$sss_repNum <- (c2-1);
      py_run_string("rng_list = [MRG32k3a(s_ss_sss_index=[s_expNum, ss, sss_repNum]) for ss in range(simulation.n_rngs)]")
      
      py_run_string("outcome=simulation.replicate(rng_list)")
      
      profit <- py$outcome[1][[1]]$profit;
      storeOutput1Sum=storeOutput1Sum+profit;
      
      #stockout=1 means sold all
      #otherwise we need to back calculate how much was unsold...
       if (py$outcome[1][[1]]$stockout==1){
         storeOutput2Sum=storeOutput2Sum+df$Input2[c1];
       }
      else{
        #let's say x was sold, "cost"  is the price per unit, a "salvage" is the salvage per unit
        # profit = (x*price) + (qty-x)*salvage- (qty*cost)
        # x*(price - salvage) = profit  + qty*(cost-salvage)
        x<- (profit+ (qty*(cost-salvage)))/(price-salvage);
        storeOutput2Sum=storeOutput2Sum+(x*100); #convert to same as input units
      }
    }
    #take mean and store in df
    df$Output1[c1] <- storeOutput1Sum/replicationNum;
    df$Output2[c1] <- storeOutput2Sum/replicationNum;
  })[3]
  df$FullEvalTime[c1] <- time;
  
  if(c1%%20==0){print(paste("Done row",c1));}
}
#rescale the profit to look proper
df$Output1 <- df$Output1*1000; #prices: *10 and quantities:*100

#Perfect Pareto Front
load("fileOfMSOprototypeFunctions")
pareto <- findParetoFront(df);
AnswerKey$df <- df;
AnswerKey$paretoFront <- pareto;
rm(df,pareto);

print("Remember to manually save the file, and make a KeyRing entry if it is a new file!");
#Save AnswerKey as a file, and all the Name and FileLocation (in a single list) into the KeyRing
# save(AnswerKey,file="AnswerKey_CNTNV");