
#MANUALLY SAVE THE ANSWERKEY AT THE END
#Setting it to autosave easily causes errors overwriting by accidentally running the file

#Each entry will need
# df : data drame of Inputs (in order), Outputs (in order) and FullEvalTime
# Name (also used as the ID)
# FullName: Optional, the full name if Name isn't clear
# ExternalProgram: Where the simulation is done
# inputNum: number of inputs
# outputNum: number of outputs
# Output_max: vector (use c( ) ) of T/F (maximize this output?)
# ParetoFront (generate this from the df)
# InputLimits (a list of vectors of the min and max one for each input --
#             Inputs MUST be integers and exactly 1 apart
#             The last value of this will not be used by the solver, remember this when setting up AnswerKey)

# Notes : Note the scaling of inputs as necessary eg: "Input1/100 is price in pounds"




rm(list=ls()) #variables



AnswerKey=list();
AnswerKey$Name <- "DualSourcing";
AnswerKey$FullName <- "Dual Sourcing System"
AnswerKey$ExternalProgram <- "SimOpt"
inputNum <- 2;
AnswerKey$inputNum <- inputNum; 
outputNum <- 3;
AnswerKey$outputNum <- outputNum;
output_max <- c(F,F,F);
AnswerKey$Output_max <-output_max;
AnswerKey$InputLimits <- list(c(30,71), c(60,101) );#remember the last entry isn't expected by solver
AnswerKey$Notes$InputNotes <- 
  "Inputs are levels that trigger an order, 1: expedited vs 2: regular";
AnswerKey$Notes$OutputNotes <- 
  "Outputs are: average ordering, average penalty cost (ran out) and average holding cost over time period. 
  These should be directly comparable (even summable)";

replicationNum <- 10; #times to simulate for 1 evaluation
AnswerKey$Notes$Replications <- replicationNum;

#prepping df
df <- expand.grid((AnswerKey$InputLimits[[1]][1]):(AnswerKey$InputLimits[[1]][2]-1),
                  (AnswerKey$InputLimits[[2]][1]):(AnswerKey$InputLimits[[2]][2]-1),KEEP.OUT.ATTRS= FALSE)

names(df)[1]="Input1";
names(df)[2]="Input2";

df[,c("Output1","Output2","Output3","FullEvalTime")]=NA;

#prep Python

library(reticulate) 

use_virtualenv("r-reticulate")
# virtualenv_list()
# py_config()

# IF THIS STOPS WORKING
# CHECK THE ASSOCIATED CONFIG FILE
# COMMENT OUT THE (unneeded "import imp" section)

#all things loaded now
py_run_string("import simopt")
py_run_string("from simopt import models, solvers, experiment_base")
py_run_string("from mrg32k3a.mrg32k3a import MRG32k3a")
py_run_string("simulation=simopt.models.dualsourcing.DualSourcing(fixed_factors=None)")

# py_run_string("print(simulation.factors)"); #to check the factors


#loop starts here

for (c1 in 1:nrow(df)){
  #for each row in the df, we need an evaluation, the mean from replicationNum trials
  
  
  
  
  time <- system.time({
    #send model inputs to python
    
    py$order_level_exp <- df$Input1[c1];
    py_run_string("simulation.factors['order_level_exp']=order_level_exp");
    py$order_level_reg <- df$Input2[c1];
    py_run_string("simulation.factors['order_level_reg']=order_level_reg");


  
    storeOutput1Sum <- 0;
    storeOutput2Sum <- 0;
    storeOutput3Sum <- 0;
    for (c2 in 1:replicationNum){
     #do replications 
      
      #set up the needed RNG 
      py$s_expNum <- (c1-1);
      py$sss_repNum <- (c2-1);
      py_run_string("rng_list = [MRG32k3a(s_ss_sss_index=[s_expNum, ss, sss_repNum]) for ss in range(simulation.n_rngs)]")
      
      py_run_string("outcome=simulation.replicate(rng_list)")
      
      #now we store the costs
      storeOutput1Sum <- storeOutput1Sum + py$outcome[[1]]$average_ordering_cost;
      storeOutput2Sum <- storeOutput2Sum + py$outcome[[1]]$average_penalty_cost;
      storeOutput3Sum <- storeOutput3Sum + py$outcome[[1]]$average_holding_cost;
    }
    #take mean and store in df
    df$Output1[c1] <- storeOutput1Sum/replicationNum;
    df$Output2[c1] <- storeOutput2Sum/replicationNum;
    df$Output3[c1] <- storeOutput3Sum/replicationNum;
  })[3]
  df$FullEvalTime[c1] <- time;
  
  if(c1%%20==0){print(paste("Done row",c1));}
}

#Perfect Pareto Front
load("fileOfMSOprototypeFunctions")
pareto <- findParetoFront(df);
AnswerKey$df <- df;
AnswerKey$paretoFront <- pareto;
rm(df,pareto);

#Save AnswerKey as a file, and all the Name and FileLocation (in a single list) into the KeyRing
# save(AnswerKey,file="???");