#basic cleaning
rm(list=ls()) #variables

#output file of functions
fileSaveName<-"fileOfMSOprototypeFunctions";

saveThis=function(filename,listToSave){
  return(c(listToSave,filename));
}

#ShortCode:ReqEval_AnyLogic
#this communicates with AnyLogic currently
#pass AL a string
reqEvaluation=function(rowIndex,df){
  con <- socketConnection(host="localhost", port = RusesPort, 
                          blocking=TRUE,
                          server=FALSE);
  
  inputNums=df[rowIndex,1:inputNum];
  stringToSend=paste(inputNums,collapse=",");
  
  #time here
  time<-system.time({
    writeLines(stringToSend,con);
    incomingString=readLines(con);
    close(con)
  })[3]
  
  qualStrings=strsplit(incomingString,",")[[1]];
  qualNums=as.numeric(qualStrings);
  
  return(append(qualNums,time));
}
listToSave<-"reqEvaluation" #the first one


#ShortCode:DominatedEval
#this takes a vector of outputs and compares to a DF of outputs
#to determine if it is pareto-dominated or not
#ie: must not be dominated in all dimensions by any option on list
#so, TRUE TRUE -> is pareto dominated, we return true
dominatedByList=function(candidateOutputs,df_paretoFront){
  dominatedOrNot<-dominatedBySingleOutput(candidateOutputs[,"Output1"],df_paretoFront[,"Output1"],output_max[1]);

  for (count in 2:outputNum)
  {
    col<-paste(c("Output",count),collapse="");
    domNext=dominatedBySingleOutput(candidateOutputs[,col],df_paretoFront[,col],output_max[count]);

    dominatedOrNot=dominatedOrNot*domNext;
  }
  if(sum(dominatedOrNot)>0)
  {
    return(T) 
  }else{
    return(F)
  }
}
listToSave=saveThis("dominatedByList",listToSave);

#this is for single output, returns a vector
dominatedBySingleOutput=function(candidateOutput,df_corresponding,wantToMax){
  if (wantToMax){
    a <- (candidateOutput<df_corresponding)
  }else{
    a <- (candidateOutput>df_corresponding)
  }
  return(a);
}
listToSave=saveThis("dominatedBySingleOutput",listToSave);


#ShortCode:FindParetoFront
#this finds a pareto front from a DF
#Works by removing from a list
findParetoFront=function(df){
  df=df[order(df[,(inputNum+1)],decreasing=T),];
  for (i in nrow(df):1)
  {
    
    
    candidateOutputs=df[i,(inputNum+1):(inputNum+outputNum)];
    df_paretoFront=df[-i,(inputNum+1):(inputNum+outputNum)];
    
    dominated<-dominatedByList(candidateOutputs,df_paretoFront)
    if(dominated)
    {
      #remove this row!
      df=df[-i,];
    }
  }
  
  #then the other way around
  df=df[nrow(df):1,];
  for (i in nrow(df):1)
  {
    
    
    candidateOutputs=df[i,(inputNum+1):(inputNum+outputNum)];
    df_paretoFront=df[-i,(inputNum+1):(inputNum+outputNum)];
    
    dominated<-dominatedByList(candidateOutputs,df_paretoFront)
    if(dominated)
    {
      #remove this row!
      df=df[-i,];
    }
  }
  
  
  
  
  return(df)
}
listToSave=saveThis("findParetoFront",listToSave);


#ShortCode:scale_df and unscale_df
scale_df=function(df,min_vector,max_vector){
  for (i in 1:ncol(df)){
    df[,i]=df[,i]-min_vector[i];
    df[,i]=df[,i]/(max_vector-min_vector)[i] ;
  }
  return(df);
}
listToSave=saveThis("scale_df",listToSave);

unscale_df=function(df,min_vector,max_vector){
  for (i in 1:ncol(df)){
    df[,i]=df[,i]*(max_vector-min_vector)[i] ;
    df[,i]=df[,i]+min_vector[i];
  }
  return(df);
}
listToSave=saveThis("unscale_df",listToSave);


#ShortCode:printFullEvals(df);
printFullEvals=function(df,msg){
  print(c(msg,"Full Evaluations:",sum(!is.na(df$FullEvalTime))));
}
listToSave=saveThis("printFullEvals",listToSave);


#takes the memory list, extracts the known points only, and trains a surrogate
#this is the selector, 
trainSurrogate=function(df,SurrogateType){
  
  #train only on Full Evaluated points
  df=df[!is.na(df$FullEvalTime),];
  
  #we need to scale etc
  df=df[,1:(inputNum+outputNum)];
  min_vector=apply(df, 2, min) 
  max_vector=apply(df, 2, max) 
  
  if (SurrogateType=="GaussianRegression")
  {
    model=trainWithGR(scale_df(df,min_vector,max_vector));
    model$SurrogateType=SurrogateType;
    model$max_vector=max_vector;
    model$min_vector=min_vector;
    return(model);
  }
  
  if (SurrogateType=="neuralnet")
  {
    model=trainWithNeuralnet(scale_df(df,min_vector,max_vector));
    model$SurrogateType=SurrogateType;
    model$max_vector=max_vector;
    model$min_vector=min_vector;
    return(model);
  }
  
  #none of the above was triggered so ERROR
  stop("The model type was not recognized, check the model's  $SurrogateType")
}
listToSave=saveThis("trainSurrogate",listToSave);

#ShortCode:TrainSurrogate_GR
trainWithGR=function(df){

  model <-gpr(input=df[,1:inputNum],response=df[,(inputNum+1):(inputNum+outputNum)], 
              Cov='matern', trace=0, useGradient=T,
              iter.max=50, nInitCandidates=50);

  #trace shows the progress for every training, not needed usually
  return(model)
}
listToSave=saveThis("trainWithGR",listToSave);

trainWithNeuralnet=function(df){
  #we need to provide a "formula" here
  string="Output1";
  for (co in 2:outputNum)
  {
   string=c(string,"+Output",co);
  }
  string=c(string," ~ Input1");
  for (co in 2:inputNum)
  {
    string=c(string,"+Input",co);
  }
  trainingFormula <- as.formula(paste(string,collapse=""));
  model <- neuralnet(trainingFormula,df,hidden=c(5,3),linear.output=F);

  return(model)
}
listToSave=saveThis("trainWithNeuralnet",listToSave);


#ShortCode:fillWithSurrogate
#fills in un-Evaluated points with the surrogate model
fillWithSurrogate=function(df,model){
  df <- arrange(df,FullEvalTime) #just in case input was not arranged
  #Selector 
  #GaussianRegression
  if(model$SurrogateType=="GaussianRegression"){
    return(fillWithGR(df,model));
  }
  
  #neuralnet
  if(model$SurrogateType=="neuralnet"){
    return(fillWithNeuralnet(df,model));
  }
  
  #none of the above was triggered so ERROR
  stop("The model type was not recognized, check the model's  $SurrogateType")
}
listToSave=saveThis("fillWithSurrogate",listToSave);

#ShortCode:EvalSurrogate_GR
fillWithGR=function(df,model){
  numKnown=sum(!is.na(df$FullEvalTime));
  needEval=df[(numKnown+1):nrow(df),1:inputNum];
  needEval=scale_df(needEval,model$min_vector,model$max_vector);
  
  inputs=data.matrix(needEval, rownames.force = NA);
  predictions=gprPredict(train=model,inputNew=inputs,noiseFreePred=T);
  
  needEval[,(inputNum+1):(inputNum+outputNum)]=predictions$pred.mean;
  needEval=unscale_df(needEval,model$min_vector,model$max_vector);
  predictions2=needEval[,(inputNum+1):(inputNum+outputNum)];
  
  df[(numKnown+1):nrow(df),(inputNum+1):(inputNum+outputNum)]=predictions2;
  return(df);
}
listToSave=saveThis("fillWithGR",listToSave);

fillWithNeuralnet=function(df,model){
  numKnown=sum(!is.na(df$FullEvalTime));
  needEval=df[(numKnown+1):nrow(df),1:(inputNum+outputNum)];
  needEval=scale_df(needEval,model$min_vector,model$max_vector);
  
  #this model already "remembers" the # of input/output 
  predictions=predict(model,needEval,rep=1,all.units=F);
  needEval[,(inputNum+1):(inputNum+outputNum)]=predictions;
  needEval=unscale_df(needEval,model$min_vector,model$max_vector);
  predictions2=needEval[,(inputNum+1):(inputNum+outputNum)];
  df[(numKnown+1):nrow(df),(inputNum+1):(inputNum+outputNum)]=predictions2;
  return(df);
}
listToSave=saveThis("fillWithNeuralnet",listToSave);


#ShortCode:fillFromAnswerKey
fillFromAnswerKey=function(df_solutionList,df_toFill, timeAllowed){
  #assumes we are using a loaded df_AnswerKey
  
  randomRows <- sample(1:nrow(df_solutionList));
  
  timeTaken <- 0;
  c1 <- 1;
  while (timeTaken < timeAllowed & c1 <= length(randomRows)){
    i <- randomRows[c1];
    
    #a is the row in the Answer Key which has what we want
    a<- df_AnswerKey[,1]==df_solutionList[i,1];
    for (c2 in 2:inputNum){
      a=a*(df_AnswerKey[,c2]==df_solutionList[i,c2]);
      
    }
    #b is the row in the df we want to full
    b<- df_toFill[,1]==df_solutionList[i,1];
    for (c3 in 2:inputNum){
      b=b*(df_toFill[,c3]==df_solutionList[i,c3]);
    }
    #update
    timeTaken <- timeTaken +df_AnswerKey[a==1,"FullEvalTime"];
    c1 <- c1+1;
    
    #only store if we would have finished
    if (timeTaken<timeAllowed){
      df_toFill[b==1,(inputNum+1):(inputNum+outputNum+1)]=df_AnswerKey[a==1,(inputNum+1):(inputNum+outputNum+1)]
    }
    
  }
  response <- list();
  response$df <- df_toFill
  
  #we may (not) have run out of time already !
  response$success <- F;
  if (c1 > length(randomRows)){
    response$success <- T;
  }
  
  return(response)   
}
listToSave=saveThis("fillFromAnswerKey",listToSave);


optimizationHandler=function(count_experiment,record_experiments)
{
  if (record_experiments$df$Optimizer[count_experiment]=="Basic")
  {
    return(optimizer_Basic(count_experiment,record_experiments));
  }
  
  if (record_experiments$df$Optimizer[count_experiment]=="GeneticAlgorithm1")
  {
    return(optimizer_GA1(count_experiment,record_experiments));
  }
  
  #none of the above was triggered so print an error
  print(paste(c("Unrecognized Optimizer, check experiment: ",count_experiment),collapse=""));
  return(record_experiments)
}
listToSave=saveThis("optimizationHandler",listToSave);

optimizer_Basic=function(count_experiment,record_experiments)
{
  #use this for storing Pareto fronts (their number may not match the row otherwise)
  ExpID <- record_experiments$df$ExpID[count_experiment];
  OptimizerStartTime=Sys.time();
  
  df_memoryOfSolutions <- setupInitialExperiments(count_experiment,record_experiments);
  df_memoryOfSolutions = arrange(df_memoryOfSolutions,FullEvalTime); #NAs are at the bottom
  FullEvalTime_soFar <- sum(df_memoryOfSolutions$FullEvalTime,na.rm=TRUE);
  record_experiments$df[count_experiment,"FullEvaluationRuns"] <- sum(!is.na(df_memoryOfSolutions$FullEvalTime));
  record_experiments$df[count_experiment,"FullEvaluationTime"] <- FullEvalTime_soFar;
  df_paretoList_soFar <- findParetoFront(df_memoryOfSolutions[!is.na(df_memoryOfSolutions$FullEvalTime),]);
  record_experiments$paretoFronts[[ExpID]]<-df_paretoList_soFar;
  record_experiments$df[count_experiment,"SurrogateRuns"]=0;
  record_experiments$df[count_experiment,"SurrogateOptimizerTime"]=0;
  
  #now we set up the surrogate model
  #this currently only has initial sample, of course
  ReqSurrogateType=record_experiments$df$SurrogateType[count_experiment];
  z2=trainSurrogate(df_memoryOfSolutions,ReqSurrogateType)
  
  #fill in all the un-Evaluated points
  df_memoryOfSolutions=fillWithSurrogate(df_memoryOfSolutions,z2);

  #find pareto front
  df_paretoList2=findParetoFront(df_memoryOfSolutions);
  
  #check/record all times etc
  num_optimizationRuns<- 1;
  OptimizerCurrentTime <- Sys.time();
  OptimizerTime_soFar <- difftime(OptimizerCurrentTime, OptimizerStartTime, units='secs');
  
  #how many on our proposed front need a full evaluation?
  num_needConfirmation <- sum(is.na(df_paretoList2$FullEvalTime));
  TimeLimit <- record_experiments$df$TimeLimit[count_experiment];

  while (num_needConfirmation>0 && ((FullEvalTime_soFar+OptimizerTime_soFar) < TimeLimit) ){
    #update surrogate stuff here (because we obviously were ok with the last run)
    record_experiments$df[count_experiment,"SurrogateRuns"]=num_optimizationRuns;
    record_experiments$df[count_experiment,"SurrogateOptimizerTime"]=OptimizerTime_soFar;
    
    
    #first we remove all evaluated members
    df_paretoList2 = df_paretoList2[is.na(df_paretoList2$FullEvalTime),];
    #now we just need to full evaluate these
    timeAllowed <- TimeLimit -FullEvalTime_soFar - OptimizerTime_soFar;
    response <- fillFromAnswerKey(df_paretoList2,df_memoryOfSolutions, timeAllowed);
    df_memoryOfSolutions = response$df;
    #pareto list recorded below, we record only after a Fill (if the fill didn't exceed the time)
    #because the fill from surrogate is not "verified"
    #if the fill was ok time-wise, but the surrogate went over time, the pareto is still recorded properly
    FullEvalTime_soFar <- sum(df_memoryOfSolutions$FullEvalTime,na.rm=TRUE);
    

    record_experiments$df[count_experiment,"FullEvaluationRuns"] <- sum(!is.na(df_memoryOfSolutions$FullEvalTime));
    record_experiments$df[count_experiment,"FullEvaluationTime"] <- FullEvalTime_soFar;
    df_paretoList_soFar <- findParetoFront(df_memoryOfSolutions[!is.na(df_memoryOfSolutions$FullEvalTime),])
    record_experiments$paretoFronts[[ExpID]]<-df_paretoList_soFar;
    
    #if we ended in time, stop here
    if (!response$success){
     break; 
    }
    

    z2=trainSurrogate(df_memoryOfSolutions,ReqSurrogateType)
    df_memoryOfSolutions=fillWithSurrogate(df_memoryOfSolutions,z2);
    df_paretoList2=findParetoFront(df_memoryOfSolutions);
    
    #update all variables
    num_needConfirmation = sum(is.na(df_paretoList2$FullEvalTime));
    
    num_optimizationRuns=num_optimizationRuns+1;
    OptimizerCurrentTime <- Sys.time();
    OptimizerTime_soFar <- difftime(OptimizerCurrentTime, OptimizerStartTime, units='secs');
    #the Pareto Front was updated ABOVE before the surrogate was trained
  }
  
  #if we actually stopped before time limit, need to record that lost last round
  if (num_needConfirmation==0 && (FullEvalTime_soFar+OptimizerTime_soFar < TimeLimit)){
    record_experiments$df[count_experiment,"SurrogateRuns"]=num_optimizationRuns;
    record_experiments$df[count_experiment,"SurrogateOptimizerTime"]=OptimizerTime_soFar;
    df_paretoList_soFar <- findParetoFront(df_memoryOfSolutions[!is.na(df_memoryOfSolutions$FullEvalTime),])
    record_experiments$paretoFronts[[ExpID]]<-df_paretoList_soFar;
  }
  
  return (record_experiments);
}
listToSave=saveThis("optimizer_Basic",listToSave);


optimizer_GA1=function(count_experiment,record_experiments){
  ExpID <- record_experiments$df$ExpID[count_experiment];
  #settings (possibly input elsewhere)
  prob_crossover=0.1;
  prob_mutation=0.1;
  
  #GA1 specific functions
  crossoverParents=function(p1,p2){
    child <- p1;
    crossed=(runif(length(p1))<prob_crossover);
    child[crossed]=p2[crossed];
    return(child)
  }
  
  mutationChild=function(soln){
    rand_draw=runif(length(soln))
    mutated=(rand_draw<prob_mutation);
    mutations=sample(c(-1,1),length(soln),replace=T);
    soln[mutated]=soln[mutated]+mutations[mutated];
    
    #enforce limits
    for (c1 in 1:length(soln)){
      soln[c1]=min(soln[c1],Limits[[c1]][2]-1);
      soln[c1]=max(soln[c1],Limits[[c1]][1]);
    }
    return(soln)
  }
  
  generateNewSolns=function(candidatePopulation){
    #we now generate new possible entities
    #only ones not known need to be kept  
    minPop <- record_experiments$df[count_experiment,"InitialSample"];
    numPop <- nrow(candidatePopulation);
    numNewSolns <- 0;
    desired <-max(2*numPop,minPop)
    limit<- 3*desired;
    while (numNewSolns<desired && limit>0){
      #pick two rows
      parentRows=sample(1:numPop,2,replace=F);
      child=crossoverParents(candidatePopulation[parentRows[1],1:inputNum],candidatePopulation[parentRows[2],1:inputNum]);
      child=mutationChild(child);
      
      if (solnIsNew(child,allKnownPopulation)&&solnIsNew(child,candidatePopulation))
      {
        #new candidate can be added !
        candidatePopulation[(nrow(candidatePopulation)+1),1:inputNum]=child;
        numNewSolns = numNewSolns+1; 
      }
      limit=limit-1;
    }
    return(candidatePopulation)
  }
  
  
  #working code starts here
  
  OptimizerStartTime=Sys.time();
  
  df_memoryOfSolutions <- setupInitialExperiments(count_experiment,record_experiments);
  df_memoryOfSolutions = arrange(df_memoryOfSolutions,FullEvalTime); #NAs are at the bottom
  FullEvalTime_soFar <- sum(df_memoryOfSolutions$FullEvalTime,na.rm=TRUE);
  record_experiments$df[count_experiment,"FullEvaluationRuns"] <- sum(!is.na(df_memoryOfSolutions$FullEvalTime));
  record_experiments$df[count_experiment,"FullEvaluationTime"] <- FullEvalTime_soFar;
  allKnownPopulation <- df_memoryOfSolutions[!is.na(df_memoryOfSolutions$FullEvalTime),];
  candidatePopulation <- findParetoFront(allKnownPopulation);
  record_experiments$paretoFronts[[ExpID]]<-candidatePopulation;
  record_experiments$df[count_experiment,"SurrogateRuns"]=0;
  record_experiments$df[count_experiment,"SurrogateOptimizerTime"]=0;
  
  #now we set up the surrogate model
  #this currently only has initial sample, of course
  ReqSurrogateType=record_experiments$df$SurrogateType[count_experiment];
  z2=trainSurrogate(df_memoryOfSolutions,ReqSurrogateType)
  
  OptimizerCurrentTime <- Sys.time();
  OptimizerTime_soFar <- difftime(OptimizerCurrentTime, OptimizerStartTime, units='secs');
  num_optimizationRuns<- 0;
  #runs is still 0 as it has not finished a single step yet
  TimeLimit <- record_experiments$df$TimeLimit[count_experiment];

  while (FullEvalTime_soFar+OptimizerTime_soFar < TimeLimit){
    #we need new population
    
    timeAllowed <- TimeLimit -FullEvalTime_soFar -OptimizerTime_soFar;
    mutationStartTime <- Sys.time();
    num_needConfirmation <- 0;
    timeTaken <- 0;
    while (num_needConfirmation == 0 & timeTaken < timeAllowed){
      candidatePopulation=generateNewSolns(candidatePopulation);
      candidatePopulation=fillWithSurrogate(candidatePopulation,z2)
      candidatePopulation <- candidatePopulation[!is.na(candidatePopulation$Input1),];
      
      #we want to keep this one into the next cycle IF there are no new possibilities
      possibleNextPopulation <- findParetoFront(candidatePopulation);
      num_needConfirmation <- sum(is.na(possibleNextPopulation$FullEvalTime));
      mutationCurrentTime <- Sys.time();
      timeTaken <- difftime(mutationCurrentTime, mutationStartTime, units='secs');
    }
    num_optimizationRuns=num_optimizationRuns+1;
    OptimizerCurrentTime <- Sys.time();
    OptimizerTime_soFar <- difftime(OptimizerCurrentTime, OptimizerStartTime, units='secs');
    record_experiments$df[count_experiment,"SurrogateRuns"]=num_optimizationRuns;
    record_experiments$df[count_experiment,"SurrogateOptimizerTime"]=OptimizerTime_soFar;
    
    if (num_needConfirmation ==0)
    {
     break; 
    }

    #now we just need to full evaluate these
    timeAllowed <- TimeLimit - FullEvalTime_soFar - OptimizerTime_soFar;
    
    response <- fillFromAnswerKey(possibleNextPopulation[is.na(possibleNextPopulation$FullEvalTime),],
                                            df_memoryOfSolutions, timeAllowed);
    df_memoryOfSolutions <- response$df;
    #pareto list recorded below, we record only after a Fill (if the fill didn't exceed the time)
    #because the fill from surrogate is not "verified"
    #if the fill was ok time-wise, but the surrogate went over time, the pareto is still recorded properly
    FullEvalTime_soFar <- sum(df_memoryOfSolutions$FullEvalTime,na.rm=TRUE);
    
    #store, then break if the time had run out already
    record_experiments$df[count_experiment,"FullEvaluationRuns"] <- sum(!is.na(df_memoryOfSolutions$FullEvalTime));
    record_experiments$df[count_experiment,"FullEvaluationTime"] <- FullEvalTime_soFar;
    allKnownPopulation <- df_memoryOfSolutions[!is.na(df_memoryOfSolutions$FullEvalTime),];
    candidatePopulation <- findParetoFront(allKnownPopulation);
    record_experiments$paretoFronts[[ExpID]]<-candidatePopulation;
    if (!response$success){
      break;
    }
    #train new model
    z2=trainSurrogate(df_memoryOfSolutions,ReqSurrogateType)
    OptimizerCurrentTime <- Sys.time();
    OptimizerTime_soFar <- difftime(OptimizerCurrentTime, OptimizerStartTime, units='secs');
  }
  
  #if we actually stopped before time limit, need to record that last surrogate round
  if (num_needConfirmation==0 && (FullEvalTime_soFar+OptimizerTime_soFar < TimeLimit)){
    record_experiments$df[count_experiment,"SurrogateRuns"]=num_optimizationRuns;
    record_experiments$df[count_experiment,"SurrogateOptimizerTime"]=OptimizerTime_soFar;
    df_paretoList_soFar <- findParetoFront(df_memoryOfSolutions[!is.na(df_memoryOfSolutions$FullEvalTime),])
    record_experiments$paretoFronts[[ExpID]]<-df_paretoList_soFar;
  }

  return(record_experiments);
}
listToSave=saveThis("optimizer_GA1",listToSave);


solnIsNew=function(soln,df){
  #note that soln is a DF with 1 row !!
  a=(df[,1]==soln[,1]);
  for (count in 2:length(soln))
  {
    a=a*(df[,count]==soln[,count])
  }
  
  if (sum(a)>0) #so it does exist
  {
    return(F); 
  }
  return(T);
}
listToSave=saveThis("solnIsNew",listToSave);


#set up the initial experiments
##======================================
setupInitialExperiments=function(count_experiment,record_experiments){
  #initial Latin Hypercube
  initialLHS <- improvedLHS(record_experiments$df$InitialSample[count_experiment], 
                            inputNum, dup = 2);
  df_initialSamples <- as.data.frame(initialLHS);
  rm(initialLHS);

  #rescale the input columns appropriately, we want from 5 to 15 (actually for all), so
  
  for (i in 1:inputNum)
  {
    #scale inputs properly
    min=Limits[[i]][1];
    max=Limits[[i]][2];
    df_initialSamples[,i]=min+floor(df_initialSamples[,i]*(max-min));
  }

  #prep the memory list
  df_memoryOfSolutions=df_AnswerKey;
  df_memoryOfSolutions[,(inputNum+1):(inputNum+outputNum+1)]=NA;
  
  #we will use the NAness of FullEvalTime and QuickEvalTime to track if we have done
  #a full eval before or not
  df_memoryOfSolutions[,"QuickEvalTime"]=NA;
  
  #this is the evaluation of initial sample
  for (i in 1:nrow(df_initialSamples)){
    a<- df_AnswerKey[,1]==df_initialSamples[i,1];
    
    for (c2 in 2:inputNum){
      a=a*(df_AnswerKey[,c2]==df_initialSamples[i,c2]);
    }

    #there should only be one
    df_memoryOfSolutions[a==1,(inputNum+1):(inputNum+outputNum+1)]=
      df_AnswerKey[a==1,(inputNum+1):(inputNum+outputNum+1)]
  }
  return(df_memoryOfSolutions);
}
listToSave=saveThis("setupInitialExperiments",listToSave);

findAnswerKey=function(UnderlyingModel,KeyRing_name){
  load(KeyRing_name);
  for (c1 in 1:length(KeyRing)){
   if (KeyRing[[c1]]$Name == UnderlyingModel){
    return(KeyRing[[c1]]$FileLocation) 
   }
    
  }
  #if not found
  stop("Error: Model not found in KeyRing");
}
listToSave=saveThis("findAnswerKey",listToSave);

save(list=listToSave,file=fileSaveName);



